/**
 * info.C: This file contains code for the NickServ INFO functions.
 *
 * Copyright © 2007 Pippijn van Steenhoven / The Ermyth Team
 * Rights to this code are as documented in COPYING.
 *
 *
 * Portions of this file were derived from sources bearing the following license:
 * Copyright © 2005-2006 William Pitcock, et al.
 * Rights to this code are as documented in doc/pod/license.pod.
 *
 * $Id: info.C,v 1.10 2007/09/22 14:27:27 pippijn Exp $
 */

#include "atheme.h"
#include <util/time.h>
#include <util/time.h>
#include <ermyth/module.h>
#include <account/mynick.h>
#include <account/myuser.h>

static char const rcsid[] = "$Id: info.C,v 1.10 2007/09/22 14:27:27 pippijn Exp $";

REGISTER_MODULE ("nickserv/info", false, "The Ermyth Team <http://ermyth.xinutec.org>");

static void ns_cmd_info (sourceinfo_t *si, int parc, char *parv[]);

command_t const ns_info = { "INFO", N_("Displays information on registrations."), AC_NONE, 2, ns_cmd_info };

E cmdvec ns_cmdtree;
E helpvec ns_helptree;

bool
_modinit (module *m)
{
  ns_cmdtree << ns_info;
  help_addentry (ns_helptree, "INFO", "help/nickserv/info", NULL);

  return true;
}

void
_moddeinit ()
{
  ns_cmdtree >> ns_info;
  help_delentry (ns_helptree, "INFO");
}

static void
ns_cmd_info (sourceinfo_t *si, int parc, char *parv[])
{
  myuser_t *mu;
  mynick_t *mn = NULL;
  user_t *u;
  char *name = parv[0];
  char buf[BUFSIZE], strfbuf[32], lastlogin[32], *p;
  struct tm tm, tm2;
  metadata *md;
  node_t *n;

  if (!name)
    {
      command_fail (si, fault::needmoreparams, STR_INSUFFICIENT_PARAMS, "INFO");
      command_fail (si, fault::needmoreparams, _("Syntax: INFO <nickname>"));
      return;
    }

  if (!(mu = myuser_t::find_ext (name)))
    {
      command_fail (si, fault::nosuch_target, _("\2%s\2 is not registered."), name);
      return;
    }

  tm = *localtime (&mu->registered);
  strftime (strfbuf, sizeof (strfbuf) - 1, "%b %d %H:%M:%S %Y", &tm);
  tm2 = *localtime (&mu->lastlogin);
  strftime (lastlogin, sizeof (lastlogin) - 1, "%b %d %H:%M:%S %Y", &tm2);

  command_success_nodata (si, _("Information on \2%s\2:"), mu->name);

  command_success_nodata (si, _("Registered: %s (%s ago)"), strfbuf, time_ago (mu->registered));
  
  if ((md = mu->find_metadata ("private:host:vhost")))
    {
      strlcpy (buf, md->value, sizeof buf);
      if ((md = mu->find_metadata ("private:usercloak")))
        {
          p = strchr (buf, '@');
          if (p == NULL)
            p = buf;
          else
            p++;
          strlcpy (p, md->value, sizeof buf - (p - buf));
        }
      command_success_nodata (si, _("Last address: %s"), buf);
    }
  
  if (has_priv (si, PRIV_USER_AUSPEX))
    {
      if ((md = mu->find_metadata ("private:host:actual")))
        command_success_nodata (si, _("Last real address: %s"), md->value);
    }
  
  if (mu->logins.empty ())
    command_success_nodata (si, _("Last seen: %s (%s ago)"), lastlogin, time_ago (mu->lastlogin));
  else if (mu == si->smu || has_priv (si, PRIV_USER_AUSPEX))
    {
      myuser_t::login_vector::iterator it, it_end;
      buf[0] = '\0';
      for (it = mu->logins.begin (), it_end = mu->logins.end (); it != it_end; ++it)
        {
          if (strlen (buf) > 80)
            {
              command_success_nodata (si, _("Logins from: %s"), buf);
              buf[0] = '\0';
            }
          if (buf[0])
            strcat (buf, " ");
          strcat (buf, (*it)->nick);
        }
      if (buf[0])
	command_success_nodata (si, _("Logins from: %s"), buf);
    }
  else
    command_success_nodata (si, _("Logins from: <hidden>"));

  if (!nicksvs.no_nick_ownership)
    {
      /* describe queried nick */
      mn = mynick_t::find (*name == '=' ? name + 1 : name);
      if (mn != NULL && mn->owner == mu)
	{
	  tm = *localtime (&mn->registered);
	  strftime (strfbuf, sizeof (strfbuf) - 1, "%b %d %H:%M:%S %Y", &tm);
	  command_success_nodata (si, _("Nick %s registered: %s (%s ago)"), mn->nick, strfbuf, time_ago (mn->registered));
	  u = user_find_named (mn->nick);
	  if (u != NULL && u->myuser == mu)
	    command_success_nodata (si, _("Nick %s is currently online"), mn->nick);
	  else
	    {
	      tm = *localtime (&mn->lastseen);
	      strftime (strfbuf, sizeof (strfbuf) - 1, "%b %d %H:%M:%S %Y", &tm);
	      command_success_nodata (si, _("Nick %s last seen: %s (%s ago)"), mn->nick, strfbuf, time_ago (mn->lastseen));
	    }
	}
      /* list registered nicks if privileged */
      if (mu == si->smu || has_priv (si, PRIV_USER_AUSPEX))
	{
	  buf[0] = '\0';
	  LIST_FOREACH (n, mu->nicks.head)
	  {
	    if (strlen (buf) > 80)
	      {
		command_success_nodata (si, _("Registered nicks: %s"), buf);
		buf[0] = '\0';
	      }
	    if (buf[0])
	      strcat (buf, " ");
	    strcat (buf, ((mynick_t *) (n->data))->nick);
	  }
	  if (buf[0])
	    command_success_nodata (si, _("Registered nicks: %s"), buf);
	}
    }

  if (!(mu->flags & MU_HIDEMAIL) || (si->smu == mu || has_priv (si, PRIV_USER_AUSPEX)))
    command_success_nodata (si, _("Email: %s%s"), mu->email, (mu->flags & MU_HIDEMAIL) ? " (hidden)" : "");

  *buf = '\0';

  if (mu->flags & MU_HIDEMAIL)
    strcat (buf, "HideMail");

  if (mu->flags & MU_HOLD)
    {
      if (*buf)
	strcat (buf, ", ");

      strcat (buf, "Hold");
    }
  if (mu->flags & MU_NEVEROP)
    {
      if (*buf)
	strcat (buf, ", ");

      strcat (buf, "NeverOp");
    }
  if (mu->flags & MU_NOOP)
    {
      if (*buf)
	strcat (buf, ", ");

      strcat (buf, "NoOp");
    }
  if (mu->flags & MU_NOMEMO)
    {
      if (*buf)
	strcat (buf, ", ");

      strcat (buf, "NoMemo");
    }
  if (mu->flags & MU_EMAILMEMOS)
    {
      if (*buf)
	strcat (buf, ", ");

      strcat (buf, "EMailMemos");
    }

  if (*buf)
    command_success_nodata (si, _("Flags: %s"), buf);

  if (mu->soper && (mu == si->smu || has_priv (si, PRIV_VIEWPRIVS)))
    {
      command_success_nodata (si, _("Oper class: %s"), mu->soper->operclass ? mu->soper->operclass->name : mu->soper->classname);
    }

  if (has_priv (si, PRIV_USER_AUSPEX) && (md = mu->find_metadata ("private:freeze:freezer")))
    {
      char const * const setter = md->value;
      time_t ts;

      md = mu->find_metadata ("private:freeze:reason");
      char const * const reason = md != NULL ? md->value : "unknown";

      md = mu->find_metadata ("private:freeze:timestamp");
      ts = md != NULL ? atoi (md->value) : 0;

      tm = *localtime (&ts);
      strftime (strfbuf, sizeof (strfbuf) - 1, "%b %d %H:%M:%S %Y", &tm);

      command_success_nodata (si, _("%s was \2FROZEN\2 by %s on %s (%s)"), mu->name, setter, strfbuf, reason);
    }

  if (has_priv (si, PRIV_USER_AUSPEX) && (md = mu->find_metadata ("private:mark:setter")))
    {
      char const * const setter = md->value;
      time_t ts;

      md = mu->find_metadata ("private:mark:reason");
      char const * const reason = md != NULL ? md->value : "unknown";

      md = mu->find_metadata ("private:mark:timestamp");
      ts = md != NULL ? atoi (md->value) : 0;

      tm = *localtime (&ts);
      strftime (strfbuf, sizeof (strfbuf) - 1, "%b %d %H:%M:%S %Y", &tm);

      command_success_nodata (si, _("%s was \2MARKED\2 by %s on %s (%s)"), mu->name, setter, strfbuf, reason);
    }

  if (mu->flags & MU_WAITAUTH)
    command_success_nodata (si, _("%s has not completed registration verification"), mu->name);

  mn->callback.info (mn, mu, si);

  command_success_nodata (si, _("*** \2End of Info\2 ***"));

  logcommand (si, CMDLOG_GET, "INFO %s", mu->name);
}
